{% for field in form %}
	<li>
		{{ field.label_tag }}
		<div>{{ field }}</div>
		{% if field.help_text %}<label class="note">{{ field.help_text }}</label>{% endif %}
	</li>
	{% if field.errors %}<div class="error_message round">{{ field.errors|join:", " }}</div>{% endif %}
{% endfor %}
