from restlib import BaseResponse, BaseClient
from filters import BasicAuthFilter, XMLParserFilter
from helpers import url_to_base_client

# *** DEPRECATED CODE ***
from deprecated import APIError, APIClient